from .utils import exceptions, objects
from .utils.generator import Generator
from .utils.headers import Headers
from .async_socket import AsyncSocket, AsyncCallBacks

from json import dumps, loads
from sys import maxsize
from random import randint
from aiohttp import ClientSession, MultipartWriter
from asyncio import get_event_loop, new_event_loop, create_task
from magic import from_buffer
from io import BytesIO
from aiofiles.threadpool.binary import AsyncBufferedReader
from typing import Union, Optional

gen = Generator()

class AsyncClient(AsyncSocket, AsyncCallBacks):
	def __init__(self, deviceId: str = None, socket_debug: bool = False, language: str = "en-US", country_code: str = "en", time_zone: int = 180):
		self.api = 'https://api.projz.com'
		self.deviceId = deviceId if deviceId else gen.deviceId()
		self.profile = objects.User()
		self.language = language
		self.country_code = country_code
		self.time_zone = time_zone

		self.session = ClientSession()

		AsyncSocket.__init__(self, headers=self.parse_headers, debug=socket_debug)
		AsyncCallBacks.__init__(self)


	def __del__(self):
		try:
			loop = get_event_loop()
			loop.run_until_complete(self.close_session())
		except RuntimeError:
			loop = new_event_loop()
			loop.run_until_complete(self.close_session())

	async def close_session(self):
		if not self.session.closed: await self.session.close()



	def parse_headers(self, endpoint: str, data = None, content_type: str = 'application/json') -> dict:
		h = Headers(deviceId=self.deviceId, sid=self.profile.sid, time_zone=self.time_zone, country_code=self.country_code, language=self.language)
		head = h.get_persistent_headers()
		head.update(h.Headers())
		head.update({"Content-Type": content_type} if content_type else {})
		head["HJTRFS"] = gen.signature(path=endpoint, headers=head, body=data or bytes())
		return head


	async def upload_media(self, file: AsyncBufferedReader, target: int = 1, returnType: str = 'object'):

		duration = 0
		file_content = await file.read()
		content = BytesIO()
		writer = MultipartWriter()
		part = writer.append(file_content, {"Content-Type": from_buffer(file_content, mime=True)})
		part.set_content_disposition("form-data", name="media", filename=file.name)
		await writer.write(objects.CopyToBufferWriter(content))
		endpoint = f"/v1/media/upload?target={target}&duration={duration}"
		async with self.session.post(f"{self.api}{endpoint}", headers=self.parse_headers(endpoint=endpoint, content_type=f"multipart/form-data; boundary={writer.boundary}", data=content.getvalue()), data=content.getvalue()) as response:
			if response.status != 200:return exceptions.CheckException(await response.text())
			else:
				return objects.Media(loads(await response.text())) if returnType == 'object' else loads(await response.text())


	async def login(self, email: str, password: str):

		data = dumps({
			"authType": 1,
			"email": email,
			"password": password
		})
		endpoint = '/v1/auth/login'

		async with self.session.post(f"{self.api}{endpoint}", headers=self.parse_headers(endpoint=endpoint, data=data), data=data) as response:
			if response.status != 200: return exceptions.CheckException(await response.text())
			else:
				self.profile = objects.User(loads(await response.text()))
				await self.connect()
				return self.profile


	async def logout(self):
		endpoint = '/v1/auth/logout'
		async with self.session.post(f"{self.api}{endpoint}", headers=self.parse_headers(endpoint=endpoint)) as response:
			if response.status != 200: return exceptions.CheckException(await response.text())
			else:
				self.profile = objects.User()
				await self.disconnect()
				return response.status

	async def Online(self):
		if self.online_loop_active: return
		self.online_loop_active = create_task(self.online_loop())
		return self.online_loop_active

	async def Offline(self):
		if self.online_loop_active:
			self.online_loop_active.cancel()
			self.online_loop_active = None
		return self.online_loop_active


	async def join_chat(self, chatId: int) -> None:

		endpoint = f'/v1/chat/threads/{chatId}/members'
		async with self.session.post(f"{self.api}{endpoint}", headers=self.parse_headers(endpoint=endpoint)) as response:
			return exceptions.CheckException(await response.text()) if response.status != 200 else response.status



	async def leave_chat(self, chatId: int) -> None:

		endpoint = f'/v1/chat/threads/{chatId}/members'
		async with self.session.delete(f"{self.api}{endpoint}", headers=self.parse_headers(endpoint=endpoint)) as response:
			return exceptions.CheckException(await response.text()) if response.status != 200 else response.status


	async def get_from_link(self, link: str):

		data = dumps({"link": link})
		endpoint = f"/v1/links/path"
		async with self.session.post(f"{self.api}{endpoint}", headers=self.parse_headers(endpoint=endpoint, data=data), data=data) as response:
			return exceptions.CheckException(await response.text()) if response.status != 200 else objects.FromLink(loads(await response.text()))


	async def get_link(self, userId: int):

		data = dumps({
			"objectId": 0,
			"objectType": 0,
			"parentId": 0,
			"path": f"user/{userId}",
			"circleIdForCircleAnnouncement": 0,
			"parentType": 0
		})

		endpoint = '/v1/links/share'
		async with self.session.post(f"{self.api}{endpoint}", headers=self.parse_headers(endpoint=endpoint, data=data), data=data) as response:
			return exceptions.CheckException(await response.text()) if response.status != 200 else objects.FromLink(loads(await response.text()))

	async def get_my_chats(self, start: int = 0, size: int = 20, type: str = 'managed'):

		endpoint = f'/v1/chat/joined-threads?start={start}&size={size}&type={type}'
		async with self.session.get(f"{self.api}{endpoint}", headers=self.parse_headers(endpoint=endpoint)) as response:
			return exceptions.CheckException(await response.text()) if response.status != 200 else  objects.Thread(loads(await response.text()))


	async def send_message(self, chatId: int, message: str, message_type: int = 1, reply_to: int = None, poll_id: int = None, dice_id: int = None): 
		data = {
			"type": message_type,
			"threadId": chatId,
			"uid": self.profile.uid,
			"seqId": randint(0, maxsize),
			"extensions": {}
		}
		data["content"] = message
		if reply_to: data["extensions"]["replyMessage"] = reply_to
		if poll_id: data["extensions"]["pollId"] = poll_id
		if dice_id: data["extensions"]["diceId"] = dice_id

		resp = await self.send(t=1, data=data, threadId=chatId)
		return resp



	async def send_verify_code(self, email: str, country_code: str = None):
		data = dumps({
			"authType": 1,
			"purpose": 1,
			"email": email,
			"password": "",
			"phoneNumber": "",
			"securityCode": "",
			"invitationCode": "",
			"secret": "",
			"gender": 0,
			"birthday": "1990-01-01",
			"requestToBeReactivated": False,
			"countryCode": country_code if country_code else self.country_code,
			"suggestedCountryCode": country_code.upper() if country_code else self.country_code.upper(),
			"ignoresDisabled": True,
			"rawDeviceIdThree": gen.generate_device_id_three()
		})
		
		endpoint = '/v1/auth/request-security-validation'
		async with self.session.post(f"{self.api}{endpoint}", headers=self.parse_headers(endpoint=endpoint, data=data), data=data) as response:
			return exceptions.CheckException(await response.text()) if response.status != 200 else response.status

	async def register(self, email: str, password: str, code: str, icon: Union[AsyncBufferedReader, objects.Media], country_code: str = None, invitation_code: str = None, nickname: str = 'XsarzyBest', tag_line: str = 'projectZ', gender: int = 100, birthday: str = '1990-01-01'):
		data = dumps({
			"authType": 1,
			"purpose": 1,
			"email": email,
			"password": password,
			"securityCode": code,
			"invitationCode": invitation_code or "",
			"nickname": nickname,
			"tagLine": tag_line,
			"icon": icon.json if isinstance(icon, objects.Media) else await self.upload_media(icon, returnType='dict'),
			"nameCardBackground": None,
			"gender": gender,
			"birthday": birthday,
			"requestToBeReactivated": False,
			"countryCode": country_code if country_code else self.country_code,
			"suggestedCountryCode": country_code.upper() if country_code else self.country_code.upper(),
			"ignoresDisabled": True,
			"rawDeviceIdThree": gen.generate_device_id_three()
		})

		endpoint = '/v1/auth/register'
		async with self.session.post(f"{self.api}{endpoint}", headers=self.parse_headers(endpoint=endpoint, data=data), data=data) as response:
			return exceptions.CheckException(await response.text()) if response.status != 200 else response.status


	async def visit(self, userId):

		endpoint = f'/v1/users/profile/{userId}/visit'
		async with self.session.post(f"{self.api}{endpoint}", headers=self.parse_headers(endpoint=endpoint)) as response:
			return exceptions.CheckException(await response.text()) if response.status != 200 else response.status


from io import BytesIO


class CopyToBufferWriter:
	def __init__(self, buffer: BytesIO):
		self.buffer = buffer

	async def write(self, data: bytes):
		self.buffer.write(data)



class Media:
	def __init__(self, data: dict = {}):
		self.json = data
		self.mediaId = self.json.get('mediaId', None)
		self.baseUrl = self.json.get('baseUrl', None)
		self.resourceList = list()

		for res in self.json.get('resourceList', []):
			self.resourceList.append(self.Resources(res))


	class Resources:
		def __init__(self, data: dict = {}):
			self.json = data
			self.width = self.json.get('width', None)
			self.height = self.json.get('height', None)
			self.url = self.json.get('url', None)
			self.thumbnail = self.json.get('thumbnail', None)



class Message:
	def __init__(self, data: dict = {}):
		self.json = data
		self.chatId = self.json.get("threadId", None)
		self.messageId = self.json.get("messageId", None)
		self.userId = self.json.get("uid", None)
		self.createdTime = self.json.get("createdTime", None)
		self.type = self.json.get("type", None)
		self.seqId = self.json.get("seqId", None)
		self.content = self.json.get("content", None)
		self.extensions = self.json.get("extensions", None)
		self.richFormat = self.json.get("richFormat", None)
		self.author = self.Author(self.json.get("richFormat", {}))

	class Author:
		def __init__(self, data: dict = {}):
			self.json = data
			self.userId = self.json.get("uid", None)
			self.nickname = self.json.get("nickname", None)
			self.gender = self.json.get("gender", None)
			self.status = self.json.get("status", None)
			self.icon = Media(self.json.get("icon", {}))
			self.extensions = self.json.get("extensions", None)


class User:
	def __init__(self, data: dict = {}):
		self.json = data
		account = self.json.get('account', {})
		userProfile = self.json.get('userProfile', {})

		self.sid = self.json.get('sId', None)
		self.secret = self.json.get('secret', None)
		self.uid = account.get('uid', None)
		self.email = account.get('email', None)
		self.createdTime = account.get('createdTime', None)
		self.deviceId = account.get('deviceId', None)
		self.googleId = account.get('googleId', None)
		self.nickname = userProfile.get('nickname', None)
		self.socialId = userProfile.get('socialId', None)
		self.gender = userProfile.get('gender', None)
		self.socialIdModified = userProfile.get('socialIdModified', None)



class UserProfile:
	def __init__(self, data: dict = {}):
		self.json=data

		extensions = self.json.get('extensions', {})
		userVisitorInfo = self.json.get('userVisitorInfo', {})

		self.userId = self.json.get('uid', None)
		self.nickname = self.json.get('nickname', None)
		self.socialId = self.json.get('socialId', None)
		self.socialIdModified = self.json.get('socialIdModified', None)
		self.gender = self.json.get('gender', None)
		self.status = self.json.get('status', None)
		self.storeId = self.json.get('storeId', None)
		self.icon = Media(self.json.get('icon', {}))
		self.chatInvitationStatus = self.json.get('chatInvitationStatus', None)
		self.publicChatInvitationStatus = self.json.get('publicChatInvitationStatus', None)
		self.privateChatInvitationStatus = self.json.get('privateChatInvitationStatus', None)
		self.circleInvitationStatus = self.json.get('circleInvitationStatus', None)
		self.adminPartyOnlineStatus = extensions.get('adminPartyOnlineStatus', None)
		self.openDaysInRow = extensions.get('openDaysInRow', None)
		self.maxOpenDaysInRow = extensions.get('maxOpenDaysInRow', None)
		self.lastOpenDate = extensions.get('lastOpenDate', None)
		self.showRecentVisitor = extensions.get('showRecentVisitor', None)
		self.onlineStatus = self.json.get('onlineStatus', None)
		self.createdTime = self.json.get('createdTime', None)
		self.contentRegion = self.json.get('contentRegion', None)
		self.contentRegionName = self.json.get('contentRegionName', None)
		self.userMood = self.UserMood(self.json.get('userMood', {}))
		self.showsSchool = self.json.get('showsSchool', None)
		self.lastActiveTime = self.json.get('lastActiveTime', None)
		self.showsLocation = self.json.get('showsLocation', None)
		self.location = self.json.get('location', None)
		self.nameCardEnabled = self.json.get('nameCardEnabled', None)
		self.matchEnabled = self.json.get('matchEnabled', None)
		self.tagline = self.json.get('tagline', None)
		self.totalViewCount = userVisitorInfo.get('totalViewCount', None)
		self.unreadViewCount = userVisitorInfo.get('unreadViewCount', None)
		self.userProfileVisitMode = userVisitorInfo.get('userProfileVisitMode', None)
		self.followMeStatus = self.json.get('followMeStatus', None)
		self.followedByMeStatus = self.json.get('followedByMeStatus', None)
		self.followedByMeStatusV2 = self.json.get('followedByMeStatusV2', None)


	class UserMood:
		def __init__(self, data: dict = {}):
			self.json = data
			self.type = self.json.get('type', None)
			self.stickerId = self.json.get('stickerId', None)
			self.text = self.json.get('text', None)
			self.onlineStatus = self.json.get('onlineStatus', None)
			self.sticker = self.Sticker(self.json.get('sticker', {}))

		class Sticker:
			def __init__(self, data: dict = {}):
				self.json = data
				self.stickerId = self.json.get('stickerId', None)
				self.name = self.json.get('name', None)
				self.media = Media(self.json.get('media', {}))









class FromLink:
	def __init__(self, data: dict = {}):
		self.json = data
		self.path = self.json.get('path', None)
		self.objectId = self.json.get('objectId', None)
		self.objectType = self.json.get('objectType', None)
		self.parentId = self.json.get('parentId', None)
		self.parentType = self.json.get('parentType', None)
		self.shareLink = self.json.get('shareLink', None)

class ThreadList:
	def __init__(self, data: dict = {}):
		self.json = data
		self.threadId = list()
		self.title = list()
		self.hostUid = list()

		for thread in self.json:
			self.threadId.append(thread.get('threadId', None))
			self.hostUid.append(thread.get('hostUid', None))
			self.title.append(thread.get('title', None))


class Thread:
	def __init__(self, data: dict = {}):
		self.json = data
		self.beMentionedThreadIdList = self.json.get('beMentionedThreadIdList', None)
		self.isEnd = self.json.get('isEnd', None)
		self.ThreadList = ThreadList(self.json.get('list', []))
		self.threadCheckList = self.json.get('threadCheckList', None)


class Event:
	def __init__(self, data: dict = {}):
		self.json = data
		msg = self.json.get('msg', {})
		author = msg.get('author', {})
		extensions = msg.get('extensions', {})
		richFormat = msg.get('richFormat', {})
		authorExtensions = author.get('extensions', {})

		self.type = self.json.get('t', None)
		self.threadId = msg.get('threadId', None)
		self.messageId = msg.get('messageId', None)
		self.userId = author.get('uid', None)
		self.createdTime = msg.get('createdTime', None)
		self.messageType = msg.get('type', None)
		self.asSummary = msg.get('asSummary', None)
		self.seqId = msg.get('seqId', None)
		self.content = msg.get('content', None)
		self.roleContentStatus = extensions.get('roleContentStatus', None)
		self.richFormatVersion = richFormat.get('version', None)
		self.nickname = author.get('nickname', None)
		self.gender = author.get('gender', None)
		self.status = author.get('status', None)
		self.icon = Media(author.get('icon', {}))
		self.adminPartyOnlineStatus = authorExtensions.get('adminPartyOnlineStatus', None)
		self.bubbleColor = msg.get('bubbleColor', None)
		self.ref = msg.get('ref', {})



class ActivateShop:
	def __init__(self, data: dict = {}):
		self.json=data

		background = self.json.get('background', {})
		extensions = self.json.get('extensions', {})

		self.storeId = self.json.get('storeId', None)
		self.storeType = self.json.get('storeType', None)
		self.contentRegion = self.json.get('contentRegion', None)
		self.storeStatus = self.json.get('storeStatus', None)
		self.status = self.json.get('status', None)
		self.name = self.json.get('name', None)
		self.description = self.json.get('description', None)
		self.icon = Media(self.json.get('icon', None))
		self.backgroundColor = background.get('backgroundColor', None)
		self.displayMode = background.get('displayMode', None)

		self.ownerId = self.json.get('ownerId', None)
		self.ownerType = self.json.get('ownerType', None)
		self.lastActivateTime = extensions.get('lastActivateTime', None)
		self.isShowDefaultIcon = extensions.get('isShowDefaultIcon', None)
		self.isShowDefaultDescription = extensions.get('isShowDefaultDescription', None)
		self.createdTime = self.json.get('createdTime', None)
		self.owner = UserProfile(self.json.get('owner', {}))



class WalletInfo:
	def __init__(self, data: dict = {}):
		self.json = data
		self.userId = self.json.get('uid', None)
		self.activateStatus = self.json.get('activateStatus', None)
		self.status = self.json.get('status', None)
		self.createdTime = self.json.get('createdTime', None)
		self.walletExtension = self.json.get('walletExtension', {})
		self.unreadTransferOrderCount = self.json.get('unreadTransferOrderCount', None)
		self.latestTransferOrder = self.LatestTransfer(self.json.get('latestTransferOrder', {}))
		self.regularAccount = self.Account(self.json.get('regularAccount', {}))
		self.storeAccount = self.Account(self.json.get('storeAccount', {}))

	class LatestTransfer:
		def __init__(self, data: dict = {}):
			self.json = data

			extensions = self.json.get('extensions', {})

			self.orderId = self.json.get('orderId', None)
			self.orderType = self.json.get('orderType', None)
			self.currencyType = self.json.get('currencyType', None)
			self.orderStatus = self.json.get('orderStatus', None)
			self.fromUid = self.json.get('fromUid', None)
			self.toUid = self.json.get('toUid', None)
			self.amount = self.json.get('amount', None)
			self.duration = self.json.get('duration', None)
			self.minClaimedTime = self.json.get('minClaimedTime', None)
			self.createdTime = self.json.get('createdTime', None)
			self.returnTime = self.json.get('returnTime', None)
			self.giftBoxId = extensions.get('status', None)
			self.expiredTime = self.json.get('expiredTime', None)
			self.currency = self.Currency(self.json.get('currency', {}))


		class Currency:
			def __init__(self, data: dict = {}):
				self.json = data
				self.name = self.json.get('name', None)
				self.symbol = self.json.get('symbol', None)
				self.icon = Media(self.json.get('icon', {}))
				self.decimals = self.json.get('decimals', None)
				self.currencyType = self.json.get('currencyType', None)
				self.amount = self.json.get('amount', None)





	class Account:
		def __init__(self, data: dict = {}):
			self.json = data
			self.accountId = self.json.get('accountId', None)
			self.accountType = self.json.get('accountType', None)
			self.status = self.json.get('status', None)
			self.createdTime = self.json.get('createdTime', None)
			self.ownerId = self.json.get('ownerId', None)
			self.ownerType = self.json.get('ownerType', None)
			self.currencyList = self.json.get('currencyList', None)




class ChatInfo:
	def __init__(self, data: dict = {}):
		self.json = data
		self.chatId = self.json.get('threadId', None)
		self.status = self.json.get('status', None)
		self.type = self.json.get('type', None)
		self.hostId = self.json.get('hostUid', None)
		self.title = self.json.get('title', None)
		self.icon = self.json.get('icon', None)
		self.content = self.json.get('content', None)
		self.latestMessageId = self.json.get('latestMessageId', None)
		self.membersCount = self.json.get('membersCount', None)
		self.allMembersCount = self.json.get('allMembersCount', None)
		self.contentRegion = self.json.get('contentRegion', None)
		self.background = Media(self.json.get('background', {}))
		self.welcomeMessage = self.json.get('welcomeMessage', None)
		self.createdTime = self.json.get('createdTime', None)
		self.updatedTime = self.json.get('updatedTime', None)
		self.language = self.json.get('language', None)
		self.visibility = self.json.get('visibility', None)
		self.rolesCount = self.json.get('rolesCount', None)
		self.qiVotedCount = self.json.get('qiVotedCount', None)
		self.usingRoleCount = self.json.get('usingRoleCount', None)
		self.talkingMemberCount = self.json.get('talkingMemberCount', None)
		self.roleplayerCount = self.json.get('roleplayerCount', None)
		self.mentionedBy = self.json.get('mentionedBy', None)
		self.plt = self.json.get('plt', None)
		self.category = self.json.get('category', None)
		self.extensions = self.json.get('extensions', None)
		self.currentMemberInfo = self.CurrentMemberInfo()
		self.latestMessage = Message(self.json.get('latestMessage', {}))
		self.host = UserProfile(self.json.get('host', {}))
		self.membersList = list()

		for member in self.json.get('membersSummary', []):
			self.membersList.append(self.UserProfile(member))


	class CurrentMemberInfo:
		def __init__(self, data: dict = {}):
			self.json = data

			extensions = self.json.get('extensions', {})

			self.chatMemberStatus = self.json.get('chatMemberStatus', None)
			self.chatMemberStatusV2 = self.json.get('chatMemberStatusV2', None)
			self.alertOption = self.json.get('alertOption', None)
			self.lastReadMessageId = self.json.get('lastReadMessageId', None)
			self.createdTime = self.json.get('createdTime', None)
			self.joinedTimeMs = self.json.get('joinedTimeMs', None)
			self.bubbleColor = extensions.get('bubbleColor', None)

class Comments:
	def __init__(self, data: dict = {}):
		self.json = data



class MentionCandidates:
	def __init__(self, data: dict = {}):
		self.json = data


class Blogs:
	def __init__(self, data: dict = {}):
		self.json = data



class BlockedUsers:
	def __init__(self, data: dict = {}):
		self.json = data




class Baners:
	def __init__(self, data: dict = {}):
		self.json = data
		self.showAtTop = self.json.get("showAtTop", None)
		self.banersList = list()
		for baner in self.json.get("list", {}):
			self.banersList.append(self.Baner(baner))


	class Baner:
		def __init__(self, data: dict = {}):
			self.json=data
			self.bannerId = self.json.get("bannerId", None)
			self.scenario = self.json.get("scenario", None)
			self.targetUrl = self.json.get("targetUrl", None)
			self.status = self.json.get("status", None)
			self.createdTime = self.json.get("createdTime", None)
			self.creatorId = self.json.get("creatorUid", None)
			self.position = self.json.get("position", None)
			self.contentRegion = self.json.get("contentRegion", None)
			self.note = self.json.get("note", None)
			self.bannerImage = Media(self.json.get("bannerImage", {}))




class Nfts:
	def __init__(self, data: dict = {}):
		self.json = data
		self.count = self.json.get("count", None)


class CirclesList:
	def __init__(self, data: dict = {}):
		self.json = data

		pagination = self.json.get("pagination", {})
		
		self.totalPages = pagination.get("total", None)
		self.nextPageToken = pagination.get("nextPageToken", None)
		self.categoryList = list()
		self.circlesList = list()

		for circle in self.json.get("list", []):
			self.circlesList.append(Circle(circle))

		for category in self.json.get("list", []):
			self.categoryList.append(Category(category))

class Circle:
	def __init__(self, data: dict = {}):
		self.json = data

		sInfo = self.json.get("sInfo", {})

		self.extensions = self.json.get("extensions", {})
		self.status = self.json.get("status", None)
		self.createdTime = self.json.get("createdTime", None)
		self.contentRegion = self.json.get("contentRegion", None)
		self.circleId = self.json.get("circleId", None)
		self.categoryId = self.json.get("categoryId", None)
		self.conceptId = self.json.get("conceptId", None)
		self.socialId = self.json.get("socialId", None)
		self.socialIdModified = self.json.get("socialIdModified", None)
		self.updatedTime = self.json.get("updatedTime", None)
		self.verifiedStatus = self.json.get("verifiedStatus", None)
		self.name = self.json.get("name", None)
		self.tagline = self.json.get("tagline", None)
		self.language = self.json.get("language", None)
		self.membersCount = self.json.get("membersCount", None)
		self.dailyActiveUser = self.json.get("dailyActiveUser", None)
		self.dailyNewPostCount = self.json.get("dailyNewPostCount", None)
		self.privacy = self.json.get("privacy", None)
		self.joinPermission = self.json.get("joinPermission", None)
		self.visibility = self.json.get("visibility", None)
		self.discoverability = self.json.get("discoverability", None)
		self.themeColor = self.json.get("themeColor", None)
		self.joinedStatus = self.json.get("joinedStatus", None)
		self.uid = self.json.get("uid", None)
		self.rootFolderId = self.json.get("rootFolderId", None)
		self.recallType = sInfo.get("recallType", None)
		self.background = Media(self.json.get("background", {}))
		self.description = self.json.get("description", None)
		self.icon = Media(self.json.get("icon", {}))
		self.cover = Media(self.json.get("cover", {}))
		self.circleIcon = Media(self.json.get("circleIcon", {}))
		self.tagList = list()

		for tag in self.json.get("tagList", []):
			self.tagList.append(Tag(tag))

class Tag:
	def __init__(self, data: dict = {}):
		self.json = data

		style = self.json.get("style", {})

		self.tagId = self.json.get("tagId", None)
		self.tagName = self.json.get("tagName", None)
		self.source = self.json.get("source", None)
		self.status = self.json.get("status", None)
		self.order = self.json.get("order", None)
		self.backgroundColor = style.get("backgroundColor", None)
		self.textColor = style.get("textColor", None)
		self.borderColor = style.get("borderColor", None)
		self.solidColor = style.get("solidColor", None)



class Category:
	def __init__(self, data: dict = {}):
		self.json = data

		sticker = self.json.get("sticker", {})
		tagInfo = self.json.get("tagInfo", {})
		tagStyle = tagInfo.get("style", {})

		self.categoryId = self.json.get("categoryId", None)
		self.stickerId = self.json.get("stickerId", None)
		self.name = self.json.get("name", None)
		self.title = self.json.get("title", None)
		self.lowerCaseName = self.json.get("lowerCaseName", None)
		self.contentRegion = self.json.get("contentRegion", None)
		self.status = self.json.get("status", None)
		self.objectType = self.json.get("objectType", None)
		self.score = self.json.get("score", None)
		self.createdTime = self.json.get("createdTime", None)
		self.conceptId = self.json.get("conceptId", None)
		self.conceptName = self.json.get("conceptName", None)
		self.categoryConcept = self.json.get("createdTime", None)
		self.extensions = self.json.get("extensions", None)
		self.stickerId = sticker.get("stickerId", None)
		self.stickerName = sticker.get("name", None)
		self.stickerMedia = Media(sticker.get("media", {}))
		self.tagId = tagInfo.get("tagId", None)
		self.tagName = tagInfo.get("tagName", None)
		self.tagTitle = tagInfo.get("title", None)
		self.tagLowerCaseName = tagInfo.get("lowerCaseName", None)
		self.languageCode = tagInfo.get("languageCode", None)
		self.tagStatus = tagInfo.get("status", None)
		self.officialVerified = tagInfo.get("officialVerified", None)
		self.tagType = tagInfo.get("tagType", None)
		self.tagCreatedTime = tagInfo.get("createdTime", None)
		self.tagBackgroundColor = tagStyle.get("backgroundColor", None)
		self.tagTextColor = tagStyle.get("textColor", None)
		self.tagBorderColor = tagStyle.get("borderColor", None)
		self.tagSolidColor = tagStyle.get("solidColor", None)


class invitationCode:
	def __init__(self, data: dict = {}):
		self.json = data
		self.userId = self.json.get("uid", None)
		self.code = self.json.get("code", None)
		self.createdTime = self.json.get("createdTime", None)
		self.status = self.json.get("status", None)
		self.label = self.json.get("label", None)
		self.generation = self.json.get("generation", None)
		self.operatorUid = self.json.get("operatorUid", None)
		self.type = self.json.get("type", None)
		self.totalUsedCount = self.json.get("totalUsedCount", None)
		self.usedCount = self.json.get("usedCount", None)


class CirclesMembers:
	def __init__(self, data: dict = {}):
		self.json = data

		pagination = self.json.get("pagination", None)

		self.nextPageToken = pagination.get("nextPageToken", None)
		self.members = list()
		for member in self.json.get("list", []):
			self.members.append(UserProfile(member))